@@ -1,7 +1,7 @@
 /*******************************************************************************
  ngx_http_h264_streaming_module.c - An Nginx module for streaming Quicktime/MPEG4 files.
 
- Copyright (C) 2008-2020 CodeShop B.V.
+ Copyright (C) 2008-2021 CodeShop B.V.
 
  For licensing see the LICENSE file
 ******************************************************************************/
@@ -22,7 +22,6 @@
 #include <fmp4/output_bucket.h>
 #include <fmp4/mp4_pubpoint.h>
 #include <fmp4/mp4_handler_io.h>
-#include <fmp4/api_process.h>
 #include <fmp4/http_util.h>
 
 #if 0
@@ -78,7 +77,7 @@ typedef struct
 
 typedef struct
 {
-  void* handler;
+  post_handler_t* handler;
 
 } ngx_http_streaming_ctx_t;
 
@@ -91,13 +90,11 @@ typedef struct
 
 static char* ngx_streaming(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static ngx_int_t ngx_streaming_handler_get(ngx_http_request_t *r,
-                                           mp4_process_context_t* context,
-                                           api_process_context_t* api_context);
+                                           mp4_process_context_t* context);
 #if defined(BUILDING_SMOOTH_STREAMING)
 static ngx_int_t ngx_http_streaming_init(ngx_conf_t* cf);
 static ngx_int_t ngx_streaming_handler_post(ngx_http_request_t *r,
-                                            mp4_process_context_t* context,
-                                            api_process_context_t* api_context);
+                                            mp4_process_context_t* context);
 static char* ngx_ism_proxy_pass(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_prefer_static(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 static char* ngx_usp_iss_pass_through(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
@@ -112,11 +109,10 @@ static char* ngx_usp_s3_region(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
 
 #endif
 static char* ngx_usp_license_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf);
-static int ngx_get_method(ngx_http_request_t* r);
+static fmp4_http_method_t ngx_get_method(ngx_http_request_t* r);
 static ngx_int_t ngx_http_add_cleanup(ngx_http_request_t* r,
                                       ngx_pool_cleanup_pt handler,
                                       void* data);
-static void ngx_http_cleanup_api_process_context(void* actx);
 static void ngx_http_cleanup_mp4_process_context(void* context);
 
 static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r);
@@ -831,9 +827,8 @@ static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r)
   context->get_server_variable_context_ = r;
   context->log_error_callback_ = log_error_callback;
   context->log_error_context_ = r;
-  context->progress_callback_ = 0;
   context->verbose_ = nxg_log_level_to_fmp4_log_level(r);
-  context->is_post_ = r->method & NGX_HTTP_POST;
+  context->method_ = ngx_get_method(r);
 
   context->prefer_static_ = slcf->usp_prefer_static == NGX_CONF_UNSET ? 0
                           : slcf->usp_prefer_static;
@@ -875,37 +870,24 @@ static ngx_int_t ngx_streaming_handler(ngx_http_request_t *r)
     }
   }
 
-  api_process_context_t* api_context = NULL;
-
   if(slcf->usp_handle_api > 0)
   {
-    ngx_int_t http_status = api_process_context_init(&api_context, context, ngx_get_method(r));
-
-    if(http_status != NGX_HTTP_OK)
-    {
-      r->headers_out.content_length_n = 0;
-      r->headers_out.status = http_status;
-      r->header_only = 1;
-      //mp4_process_context_exit(context);
-      return ngx_http_send_header(r);
-    }
-
-    ngx_http_add_cleanup(r, ngx_http_cleanup_api_process_context, api_context);
+    context->is_rest_api_ = 1;
   }
 
   if(r->method & (NGX_HTTP_GET|NGX_HTTP_HEAD))
   {
-    http_status = ngx_streaming_handler_get(r, context, api_context);
+    http_status = ngx_streaming_handler_get(r, context);
   }
 #if defined(BUILDING_SMOOTH_STREAMING)
   else if(r->method & (NGX_HTTP_POST|NGX_HTTP_PUT))
   {
-    http_status = ngx_streaming_handler_post(r, context, api_context);
+    http_status = ngx_streaming_handler_post(r, context);
   }
 
-  if(r->method & NGX_HTTP_DELETE && api_context)
+  if(r->method & NGX_HTTP_DELETE && context->is_rest_api_)
   {
-    http_status = api_process(api_context, 0, 0);
+    http_status = mp4_process(context);
     r->headers_out.content_length_n = 0;
     r->headers_out.status = http_status;
     r->header_only = 1;
@@ -974,22 +956,8 @@ static void ngx_http_fmp4_input_post_read(ngx_http_request_t *r)
   ngx_http_streaming_ctx_t* ctx =
     ngx_http_get_module_ctx(r, ngx_http_streaming_module);
 
-  post_handler_t* post_handler = NULL;
-  api_process_context_t* api_context = NULL;
-
-  if(slcf->usp_handle_api <= 0)
-  {
-    post_handler = (post_handler_t*)ctx->handler;
-
-    write_to_pubpoint(post_handler, NULL, NULL, r);
-
-    post_handler_exit(post_handler);
-  }
-  else
-  {
-    api_context = (api_process_context_t*)ctx->handler;
-    http_status = api_process(api_context, 0, 0);
-  }
+  http_status = write_to_pubpoint(ctx->handler, NULL, NULL, r);
+  post_handler_exit(ctx->handler);
 
   r->headers_out.content_length_n = 0;
   r->headers_out.status = http_status;
@@ -1013,29 +981,15 @@ ngx_http_fmp4_request_body_filter(ngx_http_request_t* r,
     ngx_http_streaming_ctx_t* ctx =
       ngx_http_get_module_ctx(r, ngx_http_streaming_module);
 
-    post_handler_t* post_handler = NULL;
-    api_process_context_t* api_context = NULL;
-
-    if(slcf->usp_handle_api <= 0)
-    {
-      post_handler = (post_handler_t*)ctx->handler;
-      ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                     "ngx_http_fmp4_request_body_filter post_handler=%p",
-                     post_handler);
-    }
-    else
-    {
-      api_context = (api_process_context_t*)ctx->handler;
-      ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                     "ngx_http_fmp4_request_body_filter api_context=%p",
-                     api_context);
-    }
+    ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
+                   "ngx_http_fmp4_request_body_filter post_handler=%p",
+                   ctx->handler);
 
     for(cl = in; cl != NULL; cl = cl->next)
     {
       ngx_int_t http_status = NGX_HTTP_OK;
 
-      if(post_handler)
+      if(ctx->handler)
       {
         size_t len = (size_t) (cl->buf->last - cl->buf->pos);
 
@@ -1044,18 +998,12 @@ ngx_http_fmp4_request_body_filter(ngx_http_request_t* r,
         if(len)
         {
           http_status =
-            write_to_pubpoint(post_handler, cl->buf->pos, cl->buf->last, r);
+            write_to_pubpoint(ctx->handler, cl->buf->pos, cl->buf->last, r);
 
           // discard the body bytes so they are not written to tmp file later
           cl->buf->pos = cl->buf->last;
         }
       }
-      else
-      {
-        http_status = api_process(api_context,
-                                  (const char*)cl->buf->pos,
-                                  (const char*)cl->buf->last);
-      }
 
       if(http_status != NGX_HTTP_OK)
       {
@@ -1074,9 +1022,7 @@ ngx_http_fmp4_request_body_filter(ngx_http_request_t* r,
 }
 
 static ngx_int_t
-ngx_streaming_handler_post(ngx_http_request_t *r,
-                           mp4_process_context_t* ctx,
-                           api_process_context_t* api_context)
+ngx_streaming_handler_post(ngx_http_request_t *r, mp4_process_context_t* ctx)
 {
   ngx_int_t http_status;
 
@@ -1089,7 +1035,7 @@ ngx_streaming_handler_post(ngx_http_request_t *r,
   char const* file = mp4_split_options_get_file(options);
 
   // Connect
-  if(!file[0] && !api_context)
+  if(!file[0] && !ctx->is_rest_api_)
   {
     r->headers_out.content_length_n = 0;
     r->headers_out.status = NGX_HTTP_OK;
@@ -1099,9 +1045,9 @@ ngx_streaming_handler_post(ngx_http_request_t *r,
   }
 
   // Pusing to .isml/Streams(...) disable api_context
-  if(mp4_starts_with(file, "Streams(") && api_context)
+  if (file[0] && ctx->is_rest_api_)
   {
-    api_context = NULL;
+    ctx->is_rest_api_ = 0;
   }
 
   if(!strcmp(file, "purge"))
@@ -1132,44 +1078,18 @@ ngx_streaming_handler_post(ngx_http_request_t *r,
                 "ngx_streaming_handler_post: manifest=%s",
                 (char const*)ctx->filename_);
 
-  post_handler_t* post_handler = NULL;
- 
-  if(!api_context)
+  post_handler_t* post_handler = pubpoint_create_post_handler(ctx);
+
+  if(post_handler == NULL)
   {
-    if(mp4_ends_with(file, ".m3u8"))
-    {
-      int is_playlist = 1;
-      post_handler = pubpoint_mpegts_stream_init(ctx, is_playlist);
-    }
-    else if(mp4_ends_with(file, ".ts"))
-    {
-      int is_playlist = 0;
-      post_handler = pubpoint_mpegts_stream_init(ctx, is_playlist);
-    }
-    else if(mp4_ends_with(file, ".mpd"))
-    {
-      // currenly we ignore MPD files.
-      r->headers_out.status = NGX_HTTP_OK;
-      r->headers_out.content_length_n = 0;
-      r->header_only = 1;
-      return ngx_http_send_header(r);
-    }
-    else
-    {
-      post_handler = pubpoint_input_stream_init(ctx);
-    }
+    ngx_log_debug(NGX_LOG_ERR, r->connection->log, 0,
+                  "usp_post_handler returned: %s", ctx->result_text_);
 
-    if(post_handler == NULL)
-    {
-      ngx_log_debug(NGX_LOG_ERR, r->connection->log, 0,
-                    "usp_post_handler returned: %s", ctx->result_text_);
-
-      return NGX_HTTP_INTERNAL_SERVER_ERROR;
-    }
+    return NGX_HTTP_INTERNAL_SERVER_ERROR;
   }
 
   ngx_log_debug(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                "ngx_streaming_handler_post: pubpoint_input_stream_init");
+                "ngx_streaming_handler_post: create_post_handler");
 
   // attach the handler to the request context to be used in the input filter
   ngx_http_streaming_ctx_t* rctx =
@@ -1183,7 +1103,7 @@ ngx_streaming_handler_post(ngx_http_request_t *r,
               "ngx_streaming_handler_post: could not create req handler ctx");
        return NGX_ERROR;
     }
-    rctx->handler = post_handler ? (void*)post_handler : (void*)api_context;
+    rctx->handler = post_handler;
     ngx_http_set_ctx(r, rctx, ngx_http_streaming_module);
   }
 
@@ -1206,8 +1126,7 @@ ngx_streaming_handler_post(ngx_http_request_t *r,
 
 static ngx_int_t
 ngx_streaming_handler_get(ngx_http_request_t *r,
-                          mp4_process_context_t* context,
-                          api_process_context_t* api_context)
+                          mp4_process_context_t* context)
 {
   ngx_int_t                   http_status;
   ngx_log_t                   *log;
@@ -1256,15 +1175,7 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
   r->root_tested = !r->error_page;
 
   {
-    if(!api_context)
-    {
-      http_status = mp4_process(context);
-    }
-    else
-    {
-      http_status = api_process(api_context, 0, 0);
-    }
-    r->headers_out.status = http_status;
+    r->headers_out.status = http_status = mp4_process(context);
 
     buckets_t* buckets = context->buckets_;
 
@@ -1773,18 +1684,18 @@ ngx_usp_license_key(ngx_conf_t* cf, ngx_command_t* cmd, void* conf)
   return NGX_CONF_OK;
 }
 
-static int ngx_get_method(ngx_http_request_t* r)
+static fmp4_http_method_t ngx_get_method(ngx_http_request_t* r)
 {
   switch (r->method)
   {
   case NGX_HTTP_GET:
-    return API_HTTP_GET;
+    return FMP4_HTTP_GET;
   case NGX_HTTP_PUT:
-    return API_HTTP_PUT;
+    return FMP4_HTTP_PUT;
   case NGX_HTTP_POST:
-    return API_HTTP_POST;
+    return FMP4_HTTP_POST;
   case NGX_HTTP_DELETE:
-    return API_HTTP_DELETE;
+    return FMP4_HTTP_DELETE;
   default:
     return r->method;
   }
@@ -1807,11 +1718,6 @@ static ngx_int_t ngx_http_add_cleanup(ngx_http_request_t* r,
   return NGX_OK;
 }
 
-static void ngx_http_cleanup_api_process_context(void* actx)
-{
-  api_process_context_exit((api_process_context_t*)actx);
-}
-
 static void ngx_http_cleanup_mp4_process_context(void* context)
 {
   mp4_process_context_exit((mp4_process_context_t*)context);
